@@ -13,8 +13,8 @@ import guanaco
 import random
 import parakeet.microscope
 import parakeet.sample
-from functools import singledispatch
 from parakeet.config import Device
+from functools import singledispatch
 
 
 __all__ = ["correct"]
@@ -30,7 +30,7 @@ def correct(
     image_file: str,
     corrected_file: str,
     num_defocus: int = 1,
-    device: Device = Device.gpu,
+    device: Device = None,
 ):
     """
     Correct the images using 3D CTF correction
@@ -49,7 +49,7 @@ def correct(
 
     # Set the device
     if device is not None:
-        config.device = device
+        config.multiprocessing.device = device
 
     # Print some options
     parakeet.config.show(config)
@@ -115,5 +115,5 @@ def _correct_Config(
         astigmatism=astigmatism,
         astigmatism_angle=astigmatism_angle,
         phase_shift=phase_shift,
-        device=config.device,
+        device=config.multiprocessing.device,
     )
