@@ -159,7 +159,8 @@ class MoleculePose(BaseModel):
     position: Optional[Tuple[float, float, float]] = Field(
         description=(
             "The molecule position (A, A, A). Setting this to null or an "
-            "empty list will cause parakeet to give a random position"
+            "empty list will cause parakeet to give a random position. "
+            "The position is given in [x y z] order. "
         ),
         examples=[
             "position: null # Assign random position",
@@ -173,7 +174,8 @@ class MoleculePose(BaseModel):
             "The molecule orientation defined as a rotation vector where "
             "the direction of the vector gives the rotation axis and the "
             "magnitude of the vector gives the rotation angle in radians. Setting "
-            "this to null or an empty list will cause parakeet to give a random orientation"
+            "this to null or an empty list will cause parakeet to give a random "
+            "orientation. The axis is given in [x, y, z] order."
         ),
         examples=[
             "orienation: null # Assign random orienation",
@@ -510,6 +512,10 @@ class Microscope(BaseModel):
         PhasePlate(), description="The phase plate parameters"
     )
 
+    objective_aperture_cutoff_freq: float = Field(
+        None, description="The objective aperture cutoff frequency (1/A)"
+    )
+
     detector: Detector = Field(Detector(), description="The detector model parameters")
 
 
@@ -671,7 +677,7 @@ class IceParameters(BaseModel):
     s2: float = Field(0.081, description="The standard deviation of gaussian 2")
     a1: float = Field(0.199, description="The amplitude of gaussian 1")
     a2: float = Field(0.801, description="The amplitude of gaussian 2")
-    density: float = Field(0.91, gt=0, description="The density of the ice (g/cm^3)")
+    density: float = Field(0.94, gt=0, description="The density of the ice (g/cm^3)")
 
 
 class Simulation(BaseModel):
@@ -721,26 +727,6 @@ class Simulation(BaseModel):
     )
 
 
-class ClusterMethod(str, Enum):
-    """
-    An enumeration to describe the cluster method
-
-    """
-
-    sge = "sge"
-
-
-class Cluster(BaseModel):
-    """
-    A model to set the cluster parameters for multiprocessing
-
-    """
-
-    method: ClusterMethod = Field(None, description="The cluster method to use")
-
-    max_workers: int = Field(1, description="The maximum number of worker processes")
-
-
 class Device(str, Enum):
     """
     An enumeration to set whether to run on the GPU or CPU
@@ -751,6 +737,19 @@ class Device(str, Enum):
     cpu = "cpu"
 
 
+class Multiprocessing(BaseModel):
+    """
+    The multiprocessing parameters
+
+    """
+
+    device: Device = Field("gpu", description="The device to use (cpu or gpu)")
+
+    nproc: int = Field(1, description="The number of processes", gt=0)
+
+    gpu_id: List[int] = Field([0], description="The GPU id for each thread")
+
+
 class Config(BaseModel):
     """
     The Parakeet configuration parameters
@@ -773,9 +772,9 @@ class Config(BaseModel):
         Simulation(), description="The simulation parameters"
     )
 
-    device: Device = Field("gpu", description="The device to use (cpu or gpu)")
-
-    cluster: Cluster = Field(Cluster(), description="The cluster parameters")
+    multiprocessing: Multiprocessing = Field(
+        Multiprocessing(), description="The multiprocessing parameters"
+    )
 
 
 def default() -> Config:
