@@ -162,7 +162,6 @@ class RelionMetadataExporter(object):
         """
 
         for i in range(1, self.config.scan.num_images + 1):
-
             # Create the dictionary
             data = {
                 "picking": pd.DataFrame.from_dict(
@@ -237,10 +236,10 @@ def export(config_file, sample_file: str, directory: str = ".", relion: bool = T
     sample = Sample(sample_file, mode="r")
 
     # Export the metadata
-    return _export_Config(config, sample)
+    return _export_Config(config, sample, directory, relion)
 
 
-@export.register
+@export.register(parakeet.config.Config)
 def _export_Config(
     config: parakeet.config.Config,
     sample: Sample,
