@@ -5,4 +5,6 @@ from parakeet.simulate._optics import * # noqa
 from parakeet.simulate._image import * # noqa
 from parakeet.simulate._ctf import * # noqa
 from parakeet.simulate._simple import * # noqa
+from parakeet.simulate._cbed import * # noqa
+import parakeet.simulate.phase_plate # noqa
 # fmt: on
